TITLE "Draws a HBITMAP on the DC"

.586
.model flat
option casemap :none      ; case sensitive
option readonly

; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
include Windows.inc
include Gdi32.inc

; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------
include asm_Graphics.inc


.data

align 16
	ptDrawOffset 	POINTL<0,0>



.code

; ----------------------------------------------------------------------------
; prototype:
;		void __stdcall DrawBitmapFromOffset(const HDC hdc, const HBITMAP hBitmap, const LPRECT pRect, const LPPOINT pOffset);
;
; Description:
;		Draws the specified bitmap on to the DC object with the correct size and
;		starting in the correct x, y offset in the rectangle.
;
; Return value:
;		(none)
; ----------------------------------------------------------------------------

; ---------------------------------------------------------------------------
; variable offests into ESP upon function entering and adjustment of the
; stack for local variables and register preservation
; ===========================================================================
; ESP + 0  = saved EBP value
; ESP + 4  = saved ESI value
; ESP + 8  = saved EDI value
; ESP + 12 = saved EBX Value
;
; ESP + 16 = (return stack)
;
; ESP + 20 = HDC parameter
; ESP + 24 = HBITMAP parameter
; ESP + 28 = LPRECT parameter
; ESP + 32 = LPPOINT parameter
align 16
_asm_DrawBitmapFromOffset@16 proc export
	; save our critical registers
	push ebx
	push esi
	push edi
	push ebp

	mov ebx, [esp + 28]
	mov ebp, [esp + 32]
	
	; create a temporary memory DC to select the bitmap into for a BitBlt to
	; the DC (which may be the screen).
	push NULL
	call CreateCompatibleDC
	mov edi, eax	; save this handle
	
	; select the HBITMAP into the DC object now for BitBlt
	push [esp + 24]
	push eax
	call SelectObject
	mov esi, eax	; save this handle

	; calculate the width and hight of the image		
	mov ecx, [ebx].RECTL.right
	mov edx, [ebx].RECTL.bottom 
	sub ecx, [ebx].RECTL.left
	sub edx, [ebx].RECTL.top
	
	; BitBlt the image now into the DC
	mov eax, [esp + 20]
	push SRCCOPY
	push [ebp].POINTL.y
	push [ebp].POINTL.x
	push edi
	push edx
	push ecx
	push [ebx].RECTL.top
	push [ebx].RECTL.left
	push eax
	call BitBlt
	
	; select the original bitmap back into the DC object so we can free the GDI resource
	push esi
	push edi
	call SelectObject
	
	; release resource associated to the DC object
	push edi
	call DeleteDC
	
	; restore the critical registers before leaving
	pop ebp
	pop edi
	pop esi
	pop ebx

	ret 16	
_asm_DrawBitmapFromOffset@16 endp


; ----------------------------------------------------------------------------
; prototype:
;		void __stdcall DrawBitmap(const HDC hdc, const HBITMAP hBitmap, const LPRECT pRect);
;
; Description:
;		Draws the specified bitmap on to the DC object with the correct size.
;
; Return value:
;		(none)
; ----------------------------------------------------------------------------

; ---------------------------------------------------------------------------
; variable offests into ESP upon function entering and adjustment of the
; stack for local variables and register preservation
; ===========================================================================
; ESP + 0 = (return stack)
;
; ESP + 4 = HDC parameter
; ESP + 8 = HBITMAP parameter
; ESP + 12 = LPRECT parameter
align 16
_asm_DrawBitmap@12 proc export
	; Call the drawing with an offset of 0,0
	push offset ptDrawOffset
	push [esp + 12 + 4]				; LPRECT parameter   edx
	push [esp +  8 + 4 + 4]			; HBITMAP parameter  ecx
	push [esp +  4 + 4 + 4 + 4]	; HDC parameter   eax
	call asm_DrawBitmapFromOffset
	
	ret 12	
_asm_DrawBitmap@12 endp

END
